import {
	UuidService,
	UuidServiceImpl,
	InvalidArgumentTypeError,
} from './uuid.service';
import { CommandHandler } from '@/packages/cli-wrapper';

export interface UuidController {
	getUsernameUuid: CommandHandler;
}

export class AdderControllerImpl implements UuidController {
	constructor(
		private readonly uuidService: UuidService = new UuidServiceImpl(),
	) {
		// noop
	}

	readonly getUsernameUuid: CommandHandler = async (params) => {
		if (!params.interactive) {
			const checkArgs = params.args as Record<string, unknown>;
			if (typeof checkArgs.username === 'undefined') {
				params.logger.error('Missing required argument: username');
				return -1;
			}
		}

		const { username, onlineMode } = params.args;
		try {
			const response = await this.uuidService.getUsernameUuid({ username, onlineMode });
			params.logger.info(response);
		} catch (errorRaw) {
			const error = errorRaw as Error;
			params.logger.error(error.message);
			if (error instanceof InvalidArgumentTypeError) {
				return -1;
			}
			return -3;
		}
		return 0;
	}
}
