import { RouteHandlerMethod } from 'fastify';
import {
	UuidService,
	UuidServiceImpl,
	InvalidArgumentTypeError,
} from '@/modules/uuid/uuid.service';
import { constants } from 'http2';

export interface UuidController {
	getUsernameUuid: RouteHandlerMethod;
}

export class UuidControllerImpl implements UuidController {
	constructor(
		private readonly uuidService: UuidService = new UuidServiceImpl(),
	) {
		// noop
	}

	readonly getUsernameUuid: RouteHandlerMethod = async (request, reply) => {
		const { username, onlineMode } = request.body as { username: string; onlineMode?: boolean };
		try {
			const response = this.uuidService.getUsernameUuid({ username, onlineMode });
			reply.send(response);
		} catch (errorRaw) {
			if (errorRaw instanceof InvalidArgumentTypeError) {
				request.log.info(errorRaw);
				reply.status(constants.HTTP_STATUS_BAD_REQUEST).send(errorRaw.message);
				return;
			}
			const error = errorRaw as Error;
			reply.status(constants.HTTP_STATUS_INTERNAL_SERVER_ERROR).send(error.message);
		}
	}
}
